/**********************************************************************
 *  MainUserDAO.java:
 **********************************************************************
 ********** author: Karan *********************************************
 **********************************************************************
 **********************************************************************/

package my.dao;

import java.util.*;
import my.hbvo.*;
import net.sf.hibernate.*;

import org.apache.log4j.*;

public class MainUserDAO implements UserDAO
{
    private static Logger log = Logger.getLogger("userManagement.dao");

    public boolean insertUser(User user) throws InfrastructureException
    {
        try
        {
            log.info("Trying to insert user: " + user.getName());
            HibernateUtil.beginTransaction();
            Session session = HibernateUtil.getSession();
            session.save(user);
            HibernateUtil.commitTransaction();
            HibernateUtil.closeSession();
            return true;
        }
        catch(Exception e)
        {
            log.error("######### Exception in insertUser(User user) ############");
            HibernateUtil.rollbackTransaction();
            HibernateUtil.closeSession();
            log.error(e);
            throw new InfrastructureException(e);
        }
    }

    public boolean updateUser(User user) throws InfrastructureException
    {
        try
        {
            log.info("Trying to update user: " + user.getName());
            HibernateUtil.beginTransaction();
            Session session = HibernateUtil.getSession();
            session.saveOrUpdate(user);
            HibernateUtil.commitTransaction();
            HibernateUtil.closeSession();
            return true;
        }
        catch(Exception e)
        {
            log.error("######### Exception in updateUser(User user) ############");
            HibernateUtil.rollbackTransaction();
            HibernateUtil.closeSession();
            log.error(e);
            throw new InfrastructureException(e);
        }
    }

    public boolean deleteUser(Long id) throws InfrastructureException
    {
        try
        {
            log.info("Trying to delete user with id: " + id);
            HibernateUtil.beginTransaction();
            Session session = HibernateUtil.getSession();
            User tmpuser = (User) session.load(User.class, id);
            session.delete(tmpuser);
            HibernateUtil.commitTransaction();
            HibernateUtil.closeSession();
            return true;
        }
        catch(Exception e)
        {
            log.error("######### Exception in deleteUser(Long id) ############");
            HibernateUtil.rollbackTransaction();
            HibernateUtil.closeSession();
            log.error(e);
            throw new InfrastructureException(e);
        }
    }


    public User getUser(Long id) throws InfrastructureException
    {
        try
        {
            log.info("Retrieving user with id: " + id);
            HibernateUtil.beginTransaction();
            Session session = HibernateUtil.getSession();
            User user = (User) session.load(User.class, id);
            HibernateUtil.commitTransaction();
            HibernateUtil.closeSession();
            if (user != null)
            {
                log.debug("Retrieved data for user with id: " + id);
                return user;
            }
            else
            {
                log.info("User with id: " + id + " does not exist.");
                return null;
            }
        }
        catch(Exception e)
        {
            log.error("######### Exception in getUser(Long id) ############");
            HibernateUtil.rollbackTransaction();
            HibernateUtil.closeSession();
            log.error(e);
            throw new InfrastructureException(e);
        }
    }

    public User getUser(String name) throws InfrastructureException
    {
        try
        {
            log.info("Retrieving user: " + name);
            HibernateUtil.beginTransaction();
            String query = "from User u where u.name=:name";
            User result = (User) HibernateUtil.getSession().createQuery(query).setString("name", name).uniqueResult();
            HibernateUtil.commitTransaction();
            HibernateUtil.closeSession();
            if (result != null)
            { 
                log.debug("Retrieved data for user with name: " + name);
                return result; 
            }
            else
            {
                log.info("User with name: " + name + " does not exist.");
                return null;
            }
        }
        catch(Exception e)
        {
            log.error("######### Exception in getUser(String name) ############");
            HibernateUtil.rollbackTransaction();
            HibernateUtil.closeSession();
            log.error(e);
            throw new InfrastructureException(e);
        }
    }

    public List getAllUsers() throws InfrastructureException
    {
        try
        {
            log.info("Retrieving the list of all users...");
            HibernateUtil.beginTransaction();
            String query = "from User u";
            List result = HibernateUtil.getSession().createQuery(query).list();
            HibernateUtil.commitTransaction();
            HibernateUtil.closeSession();
            if(result.size() > 0)
            {
                log.info("Sending back " + result.size() + " users.");
                return result;
            }
            else
            {
                log.info("No Users found.");
                return null;                    
            }
        }
        catch(Exception e)
        {
            log.error("######### Exception in getAllUsers() ############");
            HibernateUtil.rollbackTransaction();
            HibernateUtil.closeSession();
            log.error(e);
            throw new InfrastructureException(e);
        }
    }

}
