/**********************************************************************
 *  User.java:
 **********************************************************************
 ********** author: Karan *********************************************
 **********************************************************************
 **********************************************************************/

package my.hbvo;

import my.dao.*;
import java.util.*;
import org.apache.log4j.*;

public class User
{
    private Long    id              = null;
    private String  name            = null;
    private String  password        = null;
    private String  firstName       = null;
    private String  lastName        = null;
    private String  description     = null;
    private Set     userRoleSet     = new HashSet();    // FK mapping to UserRole
    
    private Set     otherRolesSet    = new HashSet();    // because they are not mapped in hibernate
    
    private static Logger log = Logger.getLogger("userManagement.hbvo");
    
    public User()
    {}

    public User(String name, String password, String firstName, String lastName, String description)
    {
        this.name         = name;
        this.password     = password;
        this.firstName    = firstName;
        this.lastName     = lastName;
        this.description  = description;
    }

    public Long getId()
    { return id; }

    public void setId(Long id)
    { this.id = id; }

    public String getName()
    { return name; }

    public void setName(String name)
    { this.name = name; }

    public String getPassword()
    { return password; }

    public void setPassword(String password)
    { this.password = password; }

    public String getFirstName()
    { return firstName; }

    public void setFirstName(String firstName)
    { this.firstName = firstName; }

    public String getLastName()
    { return lastName; }

    public void setLastName(String lastName)
    { this.lastName = lastName; }

    public String getDescription()
    { return description; }

    public void setDescription(String description)
    { this.description = description; }

    // FK mapping to UserRole
    public Set getUserRoleSet()
    { return userRoleSet; }
    
    public void setUserRoleSet(Set userRoleSet)
    { this.userRoleSet = userRoleSet; }
    
    public void addUserRole(UserRole userRole) 
    { 
        userRole.setUser(this);
        this.userRoleSet.add(userRole);
    }
    
    /*********************************** SOME BUSINESS LOGIC ***********************************************/
    public void updateRoleSets(String[] userRoles, String[] otherRoles)
    {
        // userRolesSet -> otherRolesSet
        for (int i = 0; i < otherRoles.length; i++)
        {
            Iterator userRoleSetIterator = userRoleSet.iterator();
            while(userRoleSetIterator.hasNext())
            {
                UserRole userRole = (UserRole)userRoleSetIterator.next();
                if (otherRoles[i].equals(userRole.getRole().getName()))
                {   
                    otherRolesSet.add(userRole);
                    log.debug("Added " + userRole.getRole().getName() + " to otherRolesSet.");
                    userRoleSet.remove(userRole);
                    log.debug("Removed " + userRole.getRole().getName() + " from userRoleSet.");
                }
            }
        }
        // otherRolesSet -> userRole
        for (int i = 0; i < userRoles.length; i++)
        {
            Iterator otherRolesSetIterator = otherRolesSet.iterator();
            while(otherRolesSetIterator.hasNext())
            {
                UserRole userRole = (UserRole)otherRolesSetIterator.next();
                if (userRoles[i].equals(userRole.getRole().getName()))
                {
                    userRoleSet.add(userRole);
                    log.debug("Added " + userRole.getRole().getName() + " to userRoleSet.");
                    otherRolesSet.remove(userRole);
                    log.debug("Removed " + userRole.getRole().getName() + " from otherRolesSet.");
                }
            }
        }
    }
    
    public String[] getUserRoleNames()
    {
        log.debug("Retreiving userRoleNames for user: " + this.name);
        List roleNames = new ArrayList();
        Iterator userRoleSetIterator = userRoleSet.iterator();
        while(userRoleSetIterator.hasNext())
        {
            UserRole userRole = (UserRole)userRoleSetIterator.next();
            roleNames.add(userRole.getRole().getName());
        }
        if (roleNames.size() > 0)
        {
            log.debug("Returning " + roleNames.size() + " role names.");
            String[] roleNamesArray = (String[]) roleNames.toArray(new String[0]);
            return roleNamesArray;
        }
        else
        {
            log.debug("No roles found for user: " + this.name);
            return null;
        }
    }
    
    public String[] getOtherRolesNames()
    {
        // The otherRolesSet is empty and has to be filled up manually (unlike userRoleSet)
        log.debug("Retrieving otherRolesSet from RoleDAO for user: " + this.name);
        RoleDAO roleDAO = DAOFactory.getRoleDAO();
        this.otherRolesSet = roleDAO.getOtherRolesSet(this);

        log.debug("Retreiving otherRoleNames for user: " + this.name);
        List roleNames = new ArrayList();
        Iterator otherRolesSetIterator = otherRolesSet.iterator();
        while(otherRolesSetIterator.hasNext())
        {
            Role role = (Role)otherRolesSetIterator.next();
            roleNames.add(role.getName());
        }
        if (roleNames.size() > 0)
        {
            log.debug("Returning " + roleNames.size() + " role names.");
            String[] roleNamesArray = (String[]) roleNames.toArray(new String[0]);
            return roleNamesArray;
        }
        else
        {
            log.debug("No otherRoles found for user: " + this.name);
            return null;
        }
    }
}
